/*
 * $Id: GBCamView.java,v 1.2 2002/11/25 18:51:09 dwalters Exp $
 *
 * gbcam library image viewer example for host PC
 *
 * Copyright (C) 2001 Dafydd Walters
 * dwalters@users.sourceforge.net
 *
 * Permission is hereby granted, free of charge, to use,
 * modify and distribute this example source code without
 * restriction.  Please note that such unrestricted
 * permission does NOT apply to the MotoRobots libraries
 * themselves.  The MotoRobots libraries are distributed
 * under the terms of the GNU Lesser General Public License.
 *
 * THE AUTHORS AND THE CONTRIBUTORS TO THIS WORK
 * DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE
 */

/************************************************************/
/* This program is written for Java 2 SDK (version 1.2 or   */
/* later) and requires an implementation of Java COMM API   */
/* installed on the host PC in order to run. Please see     */
/* http://www.seattlerobotics.org/encoder/200205/gbcam.html */
/* for more details.                                        */
/************************************************************/

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import javax.comm.*;
import java.util.*;

public class GBCamView extends JFrame {

  JPanel buttonPanel = new JPanel();
  JButton exposureButton = new JButton();
  JButton captureButton = new JButton();
  ImagePanel imagePanel = new ImagePanel();
  String serialPortName = null;
  SerialPort port = null;
  InputStream in = null;
  OutputStream out = null;


  public static void main(String[] args) {
    GBCamView viewer = new GBCamView();
    viewer.pack();
    viewer.show();
  }


  public GBCamView() {
    super("Camera Image Viewer");
    try {
      initGUI();
    }
    catch(Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
    loadConfig();
    promptSerialPort();
    setupPort();
  }


  private void promptSerialPort() {

    Vector v = new Vector();

    // Obtain a list of serial ports
    Enumeration en = CommPortIdentifier.getPortIdentifiers();
    while(en.hasMoreElements()) {
      CommPortIdentifier cpi = (CommPortIdentifier) en.nextElement();
      if (cpi.getPortType() == CommPortIdentifier.PORT_SERIAL)
        v.addElement(cpi.getName());
    }

    if (v.size() == 0) {
      // No serial ports found
      JOptionPane.showMessageDialog(this,
        "No serial ports found", "Error", JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }

    serialPortName = (String) JOptionPane.showInputDialog(this,
      "Please confirm serial port selection", "Camera Image Viewer",
      JOptionPane.QUESTION_MESSAGE, null, v.toArray(), serialPortName);

    if (serialPortName == null)
      System.exit(1);
    else
      saveConfig();
  }


  private void setupPort() {

    try {

      CommPortIdentifier cpi = CommPortIdentifier.getPortIdentifier(
        serialPortName);

      port = (SerialPort) cpi.open("Camera Image Viewer", 3000);
      port.setSerialPortParams(19200,
        SerialPort.DATABITS_8,
        SerialPort.STOPBITS_1,
        SerialPort.PARITY_NONE);
      port.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
      port.disableReceiveThreshold();
      port.disableReceiveTimeout();
      in = port.getInputStream();
      out = port.getOutputStream();
    } catch (Exception e) {
      JOptionPane.showMessageDialog(this,
        "Unable to setup serial port: " + e, "Error",
        JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
  }


  private void loadConfig() {
    Properties p = new Properties();
    try {
      p.load(new FileInputStream(System.getProperty("user.home") +
              System.getProperty("file.separator") + ".gbcamview.properties"));
      serialPortName =  p.getProperty("gbcamview.port");
    } catch (Exception ex) {
      serialPortName = null;
    }
  }


  private void saveConfig() {
    Properties p = new Properties();
    p.setProperty("gbcamview.port", serialPortName);
    try {
      p.store(new FileOutputStream(System.getProperty("user.home") +
              System.getProperty("file.separator") +
              ".gbcamview.properties"), "Camera Viewer Settings");
    } catch (Exception ex) {
      JOptionPane.showMessageDialog(this,
        "Unable to save settings", "Error", JOptionPane.ERROR_MESSAGE);
    }
  }


  private void initGUI() throws Exception {
    exposureButton.setText("Set Exposure");
    exposureButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        exposureButton_actionPerformed(e);
      }
    });
    captureButton.setText("Capture");
    captureButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        captureButton_actionPerformed(e);
      }
    });
    imagePanel.setBorder(BorderFactory.createEtchedBorder());
    imagePanel.setMinimumSize(new Dimension(266, 256));
    imagePanel.setPreferredSize(new Dimension(266, 256));
    this.addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        this_windowClosing(e);
      }
    });
    buttonPanel.add(captureButton, null);
    this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
    buttonPanel.add(exposureButton, null);
    this.getContentPane().add(imagePanel, BorderLayout.CENTER);
  }


  void exposureButton_actionPerformed(ActionEvent e) {
    try {
      // make sure input buffer is empty
      while (in.available() > 0)
        in.read();
      // transmit 's' command character to set exposure
      out.write('s');
      // wait for acknowledgement character
      in.read();
    } catch (IOException ex) {
      System.out.println("Exception occurred: " + ex);
    }
  }


  void captureButton_actionPerformed(ActionEvent e) {

    try {
      // make sure input buffer is empty
      while (in.available() > 0)
        in.read();
      // transmit 'p' command character to take a picture
      out.write('p');
      for (int y = 0; y < 123; y++) {
        for (int x = 0; x < 128; x++) {
          int pixel = in.read();
          if (pixel < 0 || pixel > 255)
            throw new Exception("Illegal pixel value");
          imagePanel.pixels[x][y] = pixel;
        }
      }
    } catch (Exception ex) {
      System.out.println("Exception occurred: " + ex);
    }
    // paint the image
    imagePanel.repaint();
  }


  void this_windowClosing(WindowEvent e) {
    // Window is closing, so end the application
    System.exit(0);
  }
}


// ImagePanel is a class that encapsulates the display of the image
// using an array of pixel intensities.
class ImagePanel extends JPanel {

  public int[][] pixels = new int[128][123];

  protected void paintComponent(Graphics g) {
    g.setColor(getBackground());
    g.fillRect(0, 0, getWidth(), getHeight());
    int ox = (getWidth() - 256) / 2;
    int oy = (getHeight() - 246) / 2;
    for (int y = 0; y < 123; y++) {
      for (int x = 0; x < 128; x++) {
        int pixel = pixels[x][y];
        g.setColor(new Color(pixel,pixel,pixel));
        g.fillRect(x*2 + ox, y*2 + oy, 2, 2);
      }
    }
  }
}
