/*
 * $Id: fixedpoint.h,v 1.4 2004/02/07 15:47:41 dasmith Exp $
 *
 * fixedpoint.h - Fixed point library routines to perform
 *                16.16 fixed point math.
 *
 * Copyright (C) 2003 David A. Smith
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

/**
 *
 *  @file    fixedpoint.h
 *
 *  @brief   16.16 fixed point math functions.
 *
 *  This file contains the global function prototypes,
 *  macros, and numerical constants for the <b>libfixed</b>
 *  fixed point math library.
 *
 *  To use the functions defined in this file, 
 *  <b>include <68332/fixedpoint.h></b> and link <b>libfixed.a</b>.
 *
 *  @ingroup libfixed
 */

#ifndef __FIXEDPOINT_H__
#define __FIXEDPOINT_H__

#include <68332/types.h>

#ifdef __cplusplus
extern "C" {
#endif

#define INT_TO_FIXED(x)         ((x) << 16)
#define DOUBLE_TO_FIXED(x)      ((long)((x) * 65536.0 + 0.5))
#define FIXED_TO_INT(x)         ((x) >> 16)
#define FIXED_TO_DOUBLE(x)      (((double)(x)) / 65536.0)
#define ROUND_FIXED_TO_INT(x)   (((x) + 0x8000) >> 16)

#define FIXED_ONE       INT_TO_FIXED(1)
#define FIXED_PIDIV4     51472L
#define FIXED_PIDIV2    102944L
#define FIXED_PI        205887L
#define FIXED_2PI       411775L
#define FIXED_E         178144L
#define FIXED_LN2        45426L
#define FIXED_SQRT2		 92682L

#ifndef FLOAT_PI
#define FLOAT_PI 3.141592654
#endif

/**
 * Multiplies two 16.16 fixed point numbers together,
 * returning a 16.16 fixed point product.
 * Note: No checking for overflow is performed.
 * 
 * @param  x    The first multiplicand.
 *                  
 * @param  y    The second multiplicand.
 *
 * @return      The product.
 */
int32 fpmul(int32 x, int32 y);

/**
 * Divides a 16.16 fixed point dividend by a 16.16 fixed
 * point divisor, returning a 16.16 fixed point quotient.
 * Note: No checking for underflow is performed.
 * 
 * @param  x    The dividend.
 *                  
 * @param  y    The divisor.
 *
 * @return      The quotient.
 */
int32 fpdiv(int32 x, int32 y);

/**
 * Calculates the square root of a 16.16 fixed point
 * number, returning a 16.16 fixed point result.
 * Note: No checking for underflow is performed.
 * 
 * @param  x    The input value.
 *
 * @return      The square root.
 */
int32 fpsqrt(int32 x);

/**
 * Calculates the sine of a 16.16 fixed point
 * number, returning a 16.16 fixed point result.
 * Note: The input will be transformed to within
 * the range of -FIXED_PI to +FIXED_PI.
 * 
 * @param  x    The input value in radians.
 *
 * @return      The sine of x.
 */
int32 fpsin(int32 x);

/**
 * Calculates the cosine of a 16.16 fixed point
 * number, returning a 16.16 fixed point result.
 * Note: The input will be transformed to within
 * the range of -FIXED_PI to +FIXED_PI.
 * 
 * @param  x    The input value in radians.
 *
 * @return      The cosine of x.
 */
int32 fpcos(int32 x);

/**
 * Calculates the tangent of a 16.16 fixed point
 * number, returning a 16.16 fixed point result.
 * Note: The input will be transformed to within
 * the range of -FIXED_PI to +FIXED_PI.
 * Note: The return value will be in the range of
 * -32768.0 to 32767.999984741
 * 
 * @param  x    The input value in radians.
 *
 * @return      The tangent of x.
 */
int32 fptan(int32 x);

/**
 * Calculates the arctangent of a 16.16 fixed point
 * number, returning a 16.16 fixed point result.
 * Note: The return value will be in the range of
 * -FIXED_PI/2 to FIXED_PI/2
 * 
 * @param  x    The input value.
 *
 * @return      The arctangent of x in radians.
 */
int32 fpatan(int32 x);

/**
 * Calculates the base e exponential of a 16.16 fixed
 * point number, returning a 16.16 fixed point result.
 * Note: No checking for overflow is performed.
 * 
 * @param  x    The input value.
 *
 * @return      The exponential result.
 */
int32 fpexp(int32 x);

/**
 * Calculates the base e logarithm of a 16.16 fixed
 * point number, returning a 16.16 fixed point result.
 * Note: No checking for underflow is performed.
 * 
 * @param  x    The input value.
 *
 * @return      The logarithm result.
 */
int32 fpln(int32 x);

#ifdef __cplusplus
}
#endif

#endif
