/*
 * $Id: mcpwm.h,v 1.7 2002/12/02 03:58:00 dwalters Exp $
 *
 * mcpwm.h - This file contains the definitions of the
 * functions used to manipulate the MCPWM TPU function.
 *
 * Copyright (C) 2001 Mike Panetta 
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

/**
 *
 *  @file    mcpwm.h
 *
 *  @brief   MCPWM TPU functions.
 *
 *  This file contains the global function prototypes and 
 *  data structures for the MultiChannel Pulse-Width
 *  Modulation (MCPWM) functionality of the <b>libtpu</b> library.
 *
 *  To use the functions defined in this file, 
 *  <b>include <68332/mcpwm.h></b> and link <b>libtpu.a</b>.
 *
 *  @ingroup libtpu
 */

#ifndef __TPU_MCPWM_H
#define __TPU_MCPWM_H

#include <68332/types.h>
#include <68332/tpu.h>

/**
 * Alias for the structure <b>struct _MCPWMObject</b>,
 * which represents a MCPWM object.
 */
typedef struct _MCPWMObject   MCPWMObject;

/** Alias for a pointer to <b>struct _MCPWMObject</b> */
typedef struct _MCPWMObject * MCPWMObjectP;

/** This structure represents a MCPWM object */
struct _MCPWMObject
{
	TPUChannel* masterChannel;	/**< Pointer to the master TPU channel object. */
	uint16     EAChannels;		/**< Edge-Aligned channels. */
	uint16	   CAAChannels;		/**< Ch A Center-Aligned channels. */
	uint16	   CABChannels;		/**< Ch B Center-Aligned channels. */
	uint16     priority;		/**< Priority (0-3). */
	uint16	   period;		/**< Period. */
	uint16     initialized;		/**< Initialized. */
};

/* Start of C function declarations */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * Create and initialize a new MCPWM Object
 *
 * This function creates and initializes a new MCPWM
 * TPU function.  This function only initializes the
 * master channel, use the <i>mcpwm_add...</i> functions to initialize
 * any slave channels you wish to add.
 * 
 * @param  tpu_num  The TPU to attach this function to.
 * @param  chan     The channel you wish to put the master channel on.
 * @param  priority The priority of this TPU function (1-3).
 * @param  period   The period of the PWM waveform.
 * @param  irqr	    The IRQ rate for this function (see TPUN05/D).
 *                   IRQs must be enabled for this channel for
 *                   this to have any effect.
 *                   
 * @return          MCPWM object handle.
 */
MCPWMObjectP mcpwm_init (
	uint16 tpu_num,
	uint16 chan,
	uint16 priority,
	uint16 period,
	uint16 irqr
);

/**
 * Start an MCPWM function.
 *
 * This function is used to start all the MCPWM channels
 * after they have been added with the various <i>mcpwm_add...</i>
 * functions.  This is used to ensure all the channels
 * start up at the same time.
 *
 * @param  handle  The MCPWM object you wish to start.
 *
 * @return         0 = success, -1 = failure. 
 */
int mcpwm_start (
	MCPWMObjectP handle
);

/**
 * Shutdown a MCPWM function.
 *
 * This function shuts down a MCPWM system.
 * You need to free the the MCPWMObject yourself
 * after this function returns.
 *
 * @param  handle  The MCPWM object you wish to shut down.
 */
void mcpwm_shutdown (
	MCPWMObjectP handle
);

/**
 * Add an edge-aligned TPU slave channel.
 *
 * This function allows you to create an edge-aligned
 * TPU channel associated with the master channel created
 * with the function <b>mcpwm_init()</b>.
 * 
 * @param  handle  The MCPWM object you wish to add an
 *                  edge-aligned channel to.
 * @param  chan    The channel you wish to put this on
 *                  (should be in sequence).
 * @param  high    The initial high time you wish the channel
 *                  to have when it is started.
 *
 * @return         0 = success, -1 = failure. 
 */
int mcpwm_add_ea (
	MCPWMObjectP handle,
	uint16 chan,
	uint16 high
);

/**
 * Set the high time (pulse width) for an edge-aligned channel.
 *
 * This function allows you to set the high time on an
 * edge aligned channel.
 * 
 * @param  handle  The MCPWM object you are manipulating.
 * @param  chan    The EA channel you wish to set the high time of.
 * @param  high    The value you wish to set the high time to (0-period).
 *
 * @return         0 = success, -1 = failure. 
 */
int mcpwm_set_ea_hightime (
	MCPWMObjectP handle,
	uint16 chan,
	uint16 high
);

#ifdef __cplusplus
}
#endif

#endif	// #ifndef __TPU_MCPWM_H
