/*
 * $Id: gbcam.h,v 1.8 2002/12/02 20:19:18 dwalters Exp $
 *
 * Copyright (C) 2001 Dafydd Walters
 * dwalters@users.sourceforge.net
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

/**
 *
 *  @file    gbcam.h
 *
 *  @brief   Image capture functions for the MRM.
 *
 *  This file contains the global function prototypes, constants and 
 *  data structures for libgbcam, the Game Boy Camera library.
 *
 *  For a detailed article describing how to connect a Game Boy Camera
 *  to the MRM, please refer to the web page 
 *  http://www.seattlerobotics.org/encoder/200205/gbcam.html
 *
 *  To use the functions defined in this file, 
 *  <b>include <68332/mrm/gbcam.h></b> and link <b>libgbcam.a</b>.
 *
 *  @ingroup libgbcam
 */

/**
 *  @defgroup   libgbcam   libgbcam
 *
 *  @brief   	Image capture using a hacked Game Boy Camera.
 *
 *  For a detailed article describing how to connect a Game Boy Camera
 *  to the MRM, please refer to the web page 
 *  http://www.seattlerobotics.org/encoder/200205/gbcam.html
 *
 */

#ifndef __GBCAM_H__
#define __GBCAM_H__

#include <68332/types.h>

#define gbcam_IMAGE_WIDTH   128
#define gbcam_IMAGE_HEIGHT  123
#define gbcam_IMAGE_SIZE    (gbcam_IMAGE_WIDTH * gbcam_IMAGE_HEIGHT)

/**
 * <b>gbcam_imageBuffer</b> is the image capture buffer,
 * which holds the pixel values (one byte per pixel)
 * after calling <b>gbcam_shoot()</b>.
 */
extern uint8 gbcam_imageBuffer[gbcam_IMAGE_SIZE];

/**
 * This typedef provides a convenient alias for the 
 * structure <b>struct _gbcam_RegistersType</b>, an 8-byte 
 * structure holds the register values of the
 * Mitsubishi M64282FP Artificial Retina camera chip.
 * See the M64282FP data sheet for more details. A link to
 * download the data sheet is available from the on-line
 * Encoder article at
 * http://www.seattlerobotics.org/encoder/200205/gbcam.html
 */
typedef struct _gbcam_RegistersType gbcam_RegistersType;

/**
 * This 8-byte structure holds the register values of the
 * Mitsubishi M64282FP Artificial Retina camera chip.
 * See the M64282FP data sheet for more details. A link to
 * download the data sheet is available from the on-line
 * Encoder article at
 * http://www.seattlerobotics.org/encoder/200205/gbcam.html
 */
struct _gbcam_RegistersType
{
    unsigned O  : 6;  /**< Output reference voltage */
    unsigned Z  : 2;  /**< Zero point calibration */
    unsigned G  : 5;  /**< Analog output gain */
    unsigned VH : 2;  /**< Select Vertical - Horizontal egde operation mode */
    unsigned N  : 1;  /**< Exclusively set edge enhancement mode */
    unsigned C  : 16; /**< Exposure time */
    unsigned P  : 8;  /**< 1-D filtering kernel (P) */
    unsigned M  : 8;  /**< 1-D filtering kernel (M) */
    unsigned X  : 8;  /**< 1-D filtering kernel (X) */
    unsigned V  : 3;  /**< Output mode bias voltage (Vref) */
    unsigned I  : 1;  /**< Select inverted/non-inverted output */
    unsigned E  : 4;  /**< Edge enhancement ratio */
};

/** 
 * The values in gbcam_registers can be tweaked directly,
 * if necessary, prior to calling <b>gbcam_exposure()</b> or
 * <b>gbcam_shoot()</b>.  The defaults should work fine for
 * simple image-capture applications.
 *
 * The default values of the registers are as follows:
 *
 * - O  = 0
 * - Z  = 0
 * - G  = 4
 * - VH = 0
 * - N  = 0
 * - C  = 0xAFF : this value is overwritten by gbcam_exposure()
 * - P  = 1
 * - M  = 0
 * - X  = 1
 * - V  = 7
 * - I  = 0
 * - E  = 0
 *
 * Note that the above bitfield is compiler
 * dependent (designed for GNU m68k toolchain)
 * and the defaults above correspond to the following
 * memory map:
 *
 *  .byte  0x00   ; Z1-Z0, O5-O0 <br>
 *  .byte  0x04   ; N, VH1-VH0, G4-G0 <br>
 *  .byte  0x0A   ; C17-C10 <br>
 *  .byte  0xFF   ; C07-C00 <br>
 *  .byte  0x01   ; P7-P0 <br>
 *  .byte  0x00   ; M7-M0 <br>
 *  .byte  0x01   ; X7-X0 <br>
 *  .byte  0x07   ; E3-E0, I, V2-V0 <br>
 */
extern gbcam_RegistersType gbcam_registers;

#ifdef __cplusplus
extern "C" {
#endif


/**
 * Initializes the I/O. Needs to be called once.
 */
void gbcam_init(void);

/**
 * Sets the exposure time automatically by metering
 * the image brightness several times using a binary-search
 * algorithm to maximize the mean deviation of the pixel
 * intensity values. Call this before calling <b>gbcam_shoot()</b>.
 * If you need to adjust any of the M64282FP's registers,
 * change the members of <b>gbcam_registers</b> prior to calling
 * this function. This function overwrites the value of
 * member <b>C</b> in the <b>gbcam_registers</b> structure.
 */
void gbcam_exposure(void);

/**
 * Takes a picture and stores the image in
 * the byte array <b>gbcam_imageBuffer</b>.
 */
void gbcam_shoot(void);


#ifdef __cplusplus
}
#endif

#endif /* __GBCAM_H__ */

