/*
 * $Id: ptapulse.h,v 1.1 2003/10/30 02:16:39 dasmith Exp $
 *
 * ptapulse.h - Sets up a TPU channel to measure the
 *				pulse width of an input signal.
 *
 * Copyright (C) 2003 David A. Smith
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

/**
 *
 *  @file    ptapulse.h
 *
 *  @brief   PTAPULSE TPU functions.
 *
 *  This file contains the global function prototypes and data
 *  structures for the Programmable Time Accumulator Pulse 
 *  (PTAPULSE) functionality of the <b>libtpu</b> library.
 *
 *  To use the functions defined in this file, 
 *  <b>include <68332/ptapulse.h></b> and link <b>libtpu.a</b>.
 *
 *  @ingroup libtpu
 */

#ifndef __TPU_PTAPULSE_H
#define __TPU_PTAPULSE_H

#include <68332/types.h>
#include <68332/tpu.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Starts pulse width measurement of square wave pulses input to a
 * TPU channel pin.  The first call to this function initializes
 * the pin for pulse width measurement using PTA.  Should only be
 * called once; the PTA function will measure every pulse thereafter
 * and set the channel interrupt status bit after every pulse.
 * 
 * @param  handle     The pointer to a TPUChannel returned
 *                    from tpu_request_channel().
 *
 * @param  pulseLevel The logic level of the pulse to measure.
 */
void ptapulse_init (
	TPUChannel* handle,
	bool pulseLevel
);

/**
 * Returns the result of the pulse width measurements started
 * by ptapulse_init().  Must be called after ptapulse_init().
 * Valid only when the channel interrupt status bit is set.
 * NOTE: Must be called before the next pulse measurement
 * accumulates more than 65535 counts of TCR1, or else the
 * result will be corrupted.
 * 
 * @param  handle     The pointer to a TPUChannel returned
 *                    from tpu_request_channel().
 *
 * @return            The total accumulated time for the pulse,
 *                    measured in TCR1 counts.
 */
uint32 ptapulse_result (
	TPUChannel* handle
);

#ifdef __cplusplus
}
#endif

#endif	// #ifndef __TPU_PTAPULSE_H
