/*
 * $Id: sim.h,v 1.7 2003/12/07 07:26:09 dhylands Exp $
 *
 * public interface for the 683XX system integration module (SIM)
 *
 * Copyright (C) 2001 Doug Chapman
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

/**
 *
 *  @file    sim.h
 *
 *  @brief   SIM functions.
 *
 *  This file contains the global function prototypes and 
 *  data structures for SIM support in <b>libsim</b>.
 *  
 *  To use the functions defined in this file, 
 *  <b>include <68332/sim.h></b> and link <b>libsim.a</b>.
 *  
 *  @ingroup libsim
 */

/**
 *  @defgroup   libsim	libsim
 *
 *  @brief   	System Integration Module.
 *
 */

#ifndef __SIM_H__
#define __SIM_H__

#include <68332/types.h>

/*
 * Clock speed definitions used to set processor speed.
 */
#ifndef FCRYSTAL
#define FCRYSTAL 32768L
#endif

/** 8.389 Mhz */
#define CLOCK8MHZ	0x3f00
/** 16.778 Mhz */
#define CLOCK16MHZ	0x7f00
/** 25.166 Mhz */
#define CLOCK25MHZ	0xd700	

/**
 * Possible reset states.
 */
typedef enum {
    RESET_EXT,			/**< reset via the RESET pin */
    RESET_POW,			/**< reset via power on */
    RESET_SW,			/**< software watchdog reset */
    RESET_HLT,			/**< halt monitor reset */
    RESET_SYS,			/**< reset via RESET instruction */
    RESET_TST			/**< reset via test module */
} reset_state_t;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Set the interrupt arbitration ID for the SIM.
 * 
 * @param  iarb  Valid iarb levels are 0x0 - 0xf.
 * 
 * @return       0 on success, nonzero on failure.
 */
int set_sim_iarb (
	int iarb
);

/**
 * Set processor speed.
 *
 * This function will not return until the SLOCK bit shows the synthesizer is
 * locked in on the set speed.
 *
 * @param  speed
 * It is suggested the constants CLOCK8MHZ, CLOCK16MHZ and CLOCK25MHZ
 * are used.  The upper 8-bits of this value are placed into the
 * upper 8-bits of the clock synthesizer control register.  This
 *
 * @return         0 on success, nonzero on failure.
 */
int set_processor_speed (
	int speed
);

/**
 * Get processor speed.
 *
 * This function returns the current CPU frequency (measured in cycles per second)
 * based on the settings of the SYNCR register.
 */

uint32 get_processor_freq(void);

/**
 * Query how the processor was reset.
 *
 * @return  One of the enumerated types of <b>reset_state_t</b>.  A negative
 *          return value indicates an error.
 */
reset_state_t get_reset_state(void);

/**
 * Reads IO port E.
 *
 * @return  The current 8-bit value of the PORTE data register.
 */
uint8 read_porte(void);

/**
 * Writes IO port E.
 */
void write_porte(uint8 val);

/**
 * Sets the data direction register for IO port E.
 *
 * Set the pins on port E to input or output.
 *
 * @param  direction_mask  An 8-bit value with each bit corresponding to
 *                         a pin on port E.
 *                         Setting a bit to 1 makes the pin an output.
 *                         Setting a bit to 0 makes the pin an input.
 *
 * @return                 0 on success, nonzero on error.
 */
int set_porte_data_direction (
	uint8 direction_mask
);

/**
 * Sets the pin assignment for IO port E.
 *
 * Sets pins in port E to either standard I/O, or a control
 * bus signal.  See section D.2.8 of the 68332 users guide for details.
 *
 * @param  pin_assignment  An 8-bit value with each bit corresponding 
 *                         to a pin on port E.
 * Setting a bit to 1 sets it to a control signal (as in section D.2.8).
 * Setting a bit to 0 sets it to a port E pin.
 *
 * @return                 0 on success, nonzero on error.
 */
int set_porte_pin_assignment (
	uint8 pin_assignment
);

/**
 * Reads IO port F.
 *
 * @return   The current 8-bit value of the PORTF data register.
 */
uint8 read_portf(void);

/**
 * Writes IO port F.
 */
void write_portf(uint8 val);

/**
 * Sets the data direction register for IO port F.
 *
 * Set the pins on port F to input or output.
 * @param  direction_mask  An 8-bit value with each bit corresponding to
 *                         a pin on port F.
 *                         Setting a bit to 1 makes the pin an output.
 *                         Setting a bit to 0 makes the pin an input.
 *
 * @return                 0 on success, nonzero on error.
 */
int set_portf_data_direction (
	uint8 direction_mask
);

/**
 * Set the pin assignment for IO port F.
 *
 * Sets pins in port F to either standard I/O, or a control
 * bus signal.  See section D.2.11 of the 68332 users guide for details.
 * 
 * @param  pin_assignment  An 8-bit value with each bit corresponding to a pin
 *                         on port F.
 * Setting a bit to 1 sets it to a control signal (as in section D.2.11).
 * Setting a bit to 0 sets it to a port F pin.
 *
 * @return                 0 on success, nonzero on error.
 */
int set_portf_pin_assignment (
	uint8 pin_assignment
);

/**
 * Control the system watchdog.
 *
 * Turns on/off the system watchdog.  This can be used to reset the
 * system if the watchdog timer is not reset (see <b>reset_watchdog()</b>).
 *
 * @param  enable    0 = watchdog disable.
 *                   1 = watchdog enable.
 *
 * @param  prescale  0 = watchdog clock not prescaled.
 *                   1 = watchdog clock prescaled by 512.
 *
 * @param  timeout   0 = watchdog timeout after 2^9 ticks.
 *                   1 = watchdog timeout after 2^11 ticks.
 *                   2 = watchdog timeout after 2^13 ticks.
 *                   3 = watchdog timeout after 2^15 ticks.
 *
 * @return           0 on success, nonzero on error.
 */
int set_watchdog (
	int enable, 
	int prescale, 
	int timeout
);

/**
 * Resets the watchdog timer.
 *
 * Must be called before the watchdog timer times out to prevent 
 * the system from resetting.
 *
 */
void reset_watchdog(void);

/**
 * Set up a periodic timer interrupt function.
 *
 * @param  function   C function to be called on interrupt.
 * 
 * @param  level      Interrupt request level.
 * 
 * @param  vector     Interrupt request vector.
 * 
 * @param  prescale   0 = periodic timer not prescaled.
 *                    1 = periodic timer prescaled by 512.
 *
 * @param  timer      Clock ticks to run before interrupt is triggered (8-bits).
 *
 * @return            0 on success, nonzero on failure.
 */
int periodic_interrupt_setup (
	void (*function)(),
	int level, 
	int vector,
        int prescale, 
	uint8 timer
);

/**
 * No-op routine that allows a parameter to be passed. Used to prevent
 * instruction reordering by the compiler.
 *
 * @param   val     Allows a uint16 to be read as part of this call
 */

void barrier_read_uint16( uint16 val );

/**
 * No-op routine used to prevent instruction reordering by the compiler.
 */

void barrier( void );


#ifdef __cplusplus
}
#endif

#endif /* __SIM_H__ */

