/*
 * $Id: tpu.h,v 1.15 2003/12/07 07:24:53 dhylands Exp $
 *
 * tpu.h - This file contains the definitions of the lowlevel
 * functions used to access the TPU.
 *
 * Copyright (C) 2001 Mike Panetta
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

/**
 *
 *  @file    tpu.h
 *
 *  @brief   Low-level TPU functions.
 *
 *  This file contains the global function prototypes and 
 *  data structures for the lowlevel TPU support in <b>libtpu</b>.
 *  
 *  To use the functions defined in this file, 
 *  <b>include <68332/tpu.h></b> and link <b>libtpu.a</b>.
 *  
 *  @ingroup libtpu
 */

/**
 *  @defgroup   libtpu   libtpu
 *
 *  @brief   	Time Processing Unit.
 *
 *  The <b>libtpu</b> library contains functions
 *  to drive the Motorola TPU,
 *  which is an intelligent, semi autonomous timing-control co-processor
 *  found with the core of CPUs supported by MotoRobots.
 *
 */

#ifndef __TPU_H
#define __TPU_H

#include <68332/types.h>

/** Alias for <b>struct _TPUObject</b>, representing a TPU object */
typedef struct _TPUObject   TPUObject;

/** Alias for a pointer to <b>struct _TPUObject</b> */
typedef struct _TPUObject * TPUObjectP;

/** Alias for <b>struct _TPUChannel</b>, representing a TPU channel */
typedef struct _TPUChannel   TPUChannel;

/** Alias for a pointer to <b>struct _TPUChannel</b> */
typedef struct _TPUChannel * TPUChannelP;

/**
 * TPU error codes.
 */
typedef enum
{
    TPU_ERROR_NONE = 0,
    TPU_NUMBER_OUT_OF_RANGE,
    TPU_CHANNEL_IN_USE,
    TPU_CHANNEL_OUT_OF_RANGE,
    TPU_ARGUMENT_OUT_OF_RANGE,
} tpu_errno_t;

/**
 * TPU Channel numbers
 */
typedef enum
{
	TPU_CHANNEL0 = 0,
	TPU_CHANNEL1,
	TPU_CHANNEL2,
	TPU_CHANNEL3,
	TPU_CHANNEL4,
	TPU_CHANNEL5,
	TPU_CHANNEL6,
	TPU_CHANNEL7,
	TPU_CHANNEL8,
	TPU_CHANNEL9,
	TPU_CHANNEL10,
	TPU_CHANNEL11,
	TPU_CHANNEL12,
	TPU_CHANNEL13,
	TPU_CHANNEL14,
	TPU_CHANNEL15,
} tpu_channel_t;

/**
 * TPU Priority numbers.
 */
typedef enum
{
	TPU_PRIO_NONE   = 0,
	TPU_PRIO_LOW    = 1,
	TPU_PRIO_MEDIUM = 2,
	TPU_PRIO_HIGH   = 3,
} tpu_prio_t;

/**
 * TPU Word pram slots.
 */
typedef enum
{
	TPU_PRAM_SLOT0 = 0x0,
	TPU_PRAM_SLOT1 = 0x1,
	TPU_PRAM_SLOT2 = 0x2,
	TPU_PRAM_SLOT3 = 0x3,
	TPU_PRAM_SLOT4 = 0x4,
	TPU_PRAM_SLOT5 = 0x5,
	TPU_PRAM_SLOT6 = 0x6,
	TPU_PRAM_SLOT7 = 0x7,
} tpu_pram_word_t;

/**
 * TPU Byte pram slots.
 */
typedef enum
{
	TPU_PRAM_SLOT0L = 0x0,
	TPU_PRAM_SLOT0H = 0x1,
	TPU_PRAM_SLOT1L = 0x2,
	TPU_PRAM_SLOT1H = 0x3,
	TPU_PRAM_SLOT2L = 0x4,
	TPU_PRAM_SLOT2H = 0x5,
	TPU_PRAM_SLOT3L = 0x6,
	TPU_PRAM_SLOT3H = 0x7,
	TPU_PRAM_SLOT4L = 0x8,
	TPU_PRAM_SLOT4H = 0x9,
	TPU_PRAM_SLOT5L = 0xA,
	TPU_PRAM_SLOT5H = 0xB,
	TPU_PRAM_SLOT6L = 0xC,
	TPU_PRAM_SLOT6H = 0xD,
	TPU_PRAM_SLOT7L = 0xE,
	TPU_PRAM_SLOT7H = 0xF,
} tpu_pram_byte_t;

/**
 * TPU function that the channel is currently initialized for.
 */

typedef enum
{
    TPU_FUNC_UNINITIALIZED  = 0,
    TPU_FUNC_QOMDO,
    TPU_FUNC_QOMPWM,
    TPU_FUNC_QOMPULSE,
    TPU_FUNC_FQD_A,
    TPU_FUNC_FQD_B,
    TPU_FUNC_FQDDI,
    TPU_FUNC_MCPWM_MASTER,
    TPU_FUNC_MCPWM_SLAVE,
    TPU_FUNC_PTA_0,
    TPU_FUNC_PTA_1,
    TPU_FUNC_PTA_2,
    TPU_FUNC_PTA_3,
} tpu_func_t;

/**
 * This structure represents a TPU object.
 */
struct _TPUObject {
	void *  TPUHardware;     /**< The actual pointer to the hardware itself. */
	uint16  initialized;     /**< Are we initialized yet? */
	uint16  channelsInUse;   /**< Bitmap of the channels currently being used. */
    uint32  tcr1Freq;        /**< Frequency of TCR1 clock */
    uint32  tcr2Freq;        /**< Frequency of TCR2 clock */
};

/**
 * This structure represents a TPU channel.
 */
struct _TPUChannel {
	uint16		  channel_num;  /**< Channel number (0x0 - 0xF) */
	uint16		  initialized;  /**< Initialized yet? */
	void		* TPUHardware;  /**< The actual pointer to the hardware itself. */
	tpu_errno_t	  errno;	    /**< The reason the last call failed. */
	TPUChannel 	* master;       /**< Pointer to master channel if slave. */
    void        * userData;     /**< Pointer to user defined data (for ISR) */
};


/**
 * Generate a pram address to be used as a pointer.
 *
 * This macro returns an address that can be used 
 * in the pram to refrence the pram of another channel.
 * its used for things like high_time_ptr in the mcpwm
 * function.
 * 
 * @param  c  The channel.
 * @param  p  The pram entry you want the addres of.
 */
#define PRAM_ADDRESS(c, p) ( (((c) << 4) & 0xF0) | ((p) & 0x0F) )

/**
 * Set the master channel for a particular channel
 */
#define set_channel_master(channel, master) (channel->master = master)

/**
 * Check to see if two channels share the same TPU
 */
#define channel_shares_tpu(channelA, channelB) \
	(channelA->TPUHardware == channelB->TPUHardware)

/* Start of C function declarations */
#ifdef __cplusplus
extern "C" {
#endif

/**
 * Initialize a TPU object.
 *
 * This sets up the TPU with default settings for the prescalers.
 * 
 * @param  tpu_num  Indicates which TPU to initialize; starts at 0.
 *
 * @return          0 = success, -1 = failure.
 */
int tpu_init (
	uint16 tpu_num
);

/**
 * Initialize a TPU object.
 *
 * This lets you set up every aspect of the TPU, except EMU Mode.
 * 
 * @param  tpu_num  Indicates which TPU to initialize; starts at 0.
 * @param  tcr1p    The TCR1 Prescaler settings, see pg 2-2 in the TPU RM.
 * @param  tcr2p    The TCR2 Prescaler settings, see pg 2-4 in the TPU RM.
 * @param  psck     The TPU prescaler clock setting, see pg 2-5 in the TPU RM.
 *
 * @return          0 = success, -1 = failure.
 */
int tpu_init_extended (
	uint16 tpu_num, 
	uint16 tcr1p, 
	uint16 tcr2p, 
	uint16 psck
);

/**
 * Returns the TCR1 frequency that the TPU is using.
 *
 * @param  tpu_num  Indicates which TPU to retrieve the TCR1 frequency from; starts at 0.
 *
 * @return          Frequency in Hz, 0 = failure.
 */

uint32 tpu_get_tcr1_freq( 
    uint16 tpu_num 
);

/**
 * Returns the TCR2 frequency that the TPU is using.
 *
 * @param  tpu_num  Indicates which TPU to retrieve the TCR2 frequency from; starts at 0.
 *
 * @return          Frequency in Hz, 0 = failure.
 */

uint32 tpu_get_tcr2_freq( 
    uint16 tpu_num 
);

/**
 * Initialize TPU interrupts.
 *
 * @param  tpu_num Indicates which TPU to initialize; starts at 0.
 * @param  iarb    The interrupt arbitration level for this module (0-0xF).
 * @param  cirl    The interrupt request level for all channels in the TPU.
 * @param  cibv    The interrupt base vector of the TPU.  This is the high nybble.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_init_interrupts (
	uint16 tpu_num, 
	uint16 iarb, 
	uint16 cirl, 
	uint16 cibv
);

/**
 * Request a TPU channel.
 *
 * This function lets you reserve a TPU channel for use by your function.
 * 
 * @param  tpu_num      Indicates which TPU to operate on; starts at 0.
 * @param  channel_num  The channel you wish to reserve.
 *
 * @return              TPUChannel handle.
 */
TPUChannel* tpu_request_channel (
	uint16 tpu_num, 
	uint16 channel_num
);

/**
 * Release a TPU channel.
 *
 * The function lets you notify the TPU that you are no longer using a channel.
 * 
 * @param  tpu_num      Indicates which TPU to operate on; starts at 0.
 * @param  channel_num  The channel you wish to release.
 *
 * @return              0 = success, -1 = failure.
 */
int tpu_release_channel (
	uint16 tpu_num, 
	uint16 channel_num
);
 
/**
 * Write the CIER for a particular channel.
 *
 * This function allows you to enable or disable interrupts
 * on a per channel basis.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  value   The value you wish to write. (1 enable/0 disable).
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_set_CIER (
	TPUChannel* handle, 
	uint16 value
);

/**
 * Read the CISR for a particular channel.
 *
 * This function allows you to get the interrupt status of a channel.  
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * 
 * @return         The CISR value for this channel, or -1 on error.
 */
int tpu_get_CISR (
	TPUChannel* handle
);

/**
 * Clear the CISR for a particular channel.
 *
 * This function allows you to clear the interrupt status of a channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_clear_CISR (
	TPUChannel* handle
);

/**
 * Write the CFSR for a particular channel.
 *
 * This function is used to set the CFSR for a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  function_code  The function number you wish to set.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_set_CFSR (
	TPUChannel* handle, 
	uint16 function_code
);

/**
 * Read the CFSR for a particular channel.
 *
 * This function is used to get the CFSR for a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * 
 * @return         The function code for this channel, or -1 on error.
 */
int tpu_get_CFSR (
	TPUChannel* handle
);

/**
 * Write the HSQR for a particular channel.
 *
 * This function is used to set the HSQR for a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  value   The host sequence number you wish to set.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_set_HSQR (
	TPUChannel* handle, 
	uint16 value
);

/**
 * Read the HSQR for a particular channel.
 *
 * This function is used to get the HSQR for a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * 
 * @return         The host sequence number for this channel, or -1 on error.
 */
int tpu_get_HSQR (
	TPUChannel* handle
);

/**
 * Write the HSRR for a particular channel.
 *
 * This function is used to set the HSRR for a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  value   The host request you wish to send.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_set_HSRR (
	TPUChannel* handle, 
	uint16 value
);

/**
 * Read the HSRR for a particular channel.
 *
 * This function is used to get the HSRR for a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * 
 * @return         The host request number for this channel, or -1 on error.
 */
int tpu_get_HSRR (
	TPUChannel* handle
);

/**
 * Write the CPR for a particular channel.
 *
 * This function is used to set the CPR for a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  value   The priority you wish to set.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_set_CPR (
	TPUChannel* handle, 
	uint16 value
);

/**
 * Read the CPR for a particular channel.
 *
 * This function is used to set the CPR for a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * 
 * @return         The channel priority for this channel, -1 on error.
 */
int tpu_get_CPR (
	TPUChannel* handle
);

/**
 * Write the parameter ram (as word) for a particular channel.
 *
 * This function is used to write a parameter ram entry for
 * a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  entry   The parameter ram entry you are modifying. (0-E)
 * @param  value   The value you wish to write.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_write_parameter_word (
	TPUChannel* handle,
	tpu_pram_word_t entry, 
	uint16 value
);

/**
 * Write the parameter ram (as byte) for a particular channel.
 *
 * This function is used to write a parameter ram entry for
 * a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  entry   The parameter ram entry you are modifying. (0-F)
 * @param  value   The value you wish to write.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_write_parameter_byte (
	TPUChannel* handle,
	tpu_pram_byte_t entry, 
	uint8 value
);

/**
 * Read the parameter ram (as word) for a particular channel.
 *
 * This function is used to read a parameter ram entry for
 * a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  entry   The parameter ram entry you are modifying. (0-E)
 * @param  value   Pointer to where you wish to store the value read.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_read_parameter_word (
	TPUChannel* handle,
	tpu_pram_word_t entry,
	uint16* value
);

/**
 * Read the parameter ram (as byte) for a particular channel.
 *
 * This function is used to read a parameter ram entry for
 * a particular channel.
 * 
 * @param  handle  A pointer to the TPUChannel that you are operating on.
 * @param  entry   The parameter ram entry you are modifying (0-7).
 * @param  value   Pointer to where you wish to store the value read.
 *
 * @return         0 = success, -1 = failure.
 */
int tpu_read_parameter_byte ( 
	TPUChannel* handle,
	tpu_pram_byte_t entry, 
	uint8* value
);

/**
 * Print the error that last occured on the TPU.
 *
 * This function is used to print the error that last
 * occured as a result of calling one of the TPU functions.
 * This function only prints things when DEBUG is defined.
 *
 * @param  handle  The TPUObject you are operating on.
 */
void tpu_perror ( 
	TPUChannel* handle
);

#ifdef __cplusplus
}
#endif

#endif	/* __TPU_H */

// vim: ts=4
// vim: sw=4
