/*
 * $Id: fixedpoint.c,v 1.3 2004/01/29 03:04:32 dasmith Exp $
 *
 * fixedpoint.c - Fixed point library routines to perform
 *                16.16 fixed point math.
 *
 * Copyright (C) 2003 David A. Smith
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

#include <68332/fixedpoint.h>

#if !defined( PORTABLE )
#define PORTABLE 0
#endif

#if PORTABLE

int32 fpmul(int32 x, int32 y)
{
  long long z = (long long)x * (long long)y;
  return ((int)(z >> 16));
}

int32 fpdiv(int32 x, int32 y)
{
//  long long z = (((long long)x) << 32);
//  return (int32)((z / y) >> 16);
  long long z = (((long long)x) << 16);
  return (int32)(z / y);
}

#else

int32 fpmul(int32 x, int32 y)
{
  asm volatile ("
  move.l  %d1, -(%a7)
  move.l 8(%a6), %d0
  move.l 12(%a6), %d1
  muls.l %d1,%d1:%d0
  move.w %d1,%d0
  swap   %d0
  move.l  (%a7)+, %d1
  ");
}

int32 fpdiv(int32 x, int32 y)
{
  asm volatile ("
  move.l  %d2, -(%a7)
  move.l  %d1, -(%a7)
  move.l 8(%a6), %d2
  move.l 12(%a6), %d1
  beq.s  fpdiv_error
  move.w %d2, %d0
  swap   %d0
  clr.w  %d0
  swap   %d2
  ext.l  %d2
  divs.l %d1,%d2:%d0
  bra.s  fpdiv_end
fpdiv_error:
  move.l #-2, %d0
  ror    #1, %d0
  tst.l  %d2
  bgt    fpdiv_end
  not.l  %d0
fpdiv_end:
  move.l  (%a7)+, %d1
  move.l  (%a7)+, %d2
  ");
}

#endif

int32 fpsqrt(int32 x)
{
  register int32 y = x;
  register int32 a = 0;

  if (y <= 0)
  {
    return 0;
  }
  else if (y > FIXED_ONE)
  {
    while (y & 0xFFFF0000)
    {
      y >>= 1;
      a++;
    }
  }
  else
  {
    while (!(y & 0xFFFF0000))
    {
      y <<= 1;
      a--;
    }

    y >>= 1;
    a++;
  }

  y = (DOUBLE_TO_FIXED(0.95) + y) >> 1;

  if (a & 0x00000001) // a is odd
  {
    a = a >> 1;

    if (a > 0)
    {
      y = fpmul(fpmul(y, FIXED_ONE << a), FIXED_SQRT2);
    }
    else
    {
      y = fpmul(fpmul(y, FIXED_ONE >> -a), FIXED_SQRT2);
    }
  }
  else // a is even
  {
    a = a >> 1;

    if (a > 0)
    {
      y = fpmul(y, FIXED_ONE << a);
    }
    else
    {
      y = fpmul(y, FIXED_ONE >> -a);
    }
  }

  // Iterate two times to improve accuracy.
  // One of the below two lines may be removed to
  // increase speed but at the cost of reduced accuracy.
  y = (y + fpdiv(x, y)) >> 1;
  y = (y + fpdiv(x, y)) >> 1;

  return y;  
}

int32 fpsin(int32 x)
{
  register int32 x2;
  register int32 numerator;
  register int32 denominator;

  while (x > FIXED_PI)
    x -= FIXED_2PI;

  while (x < -FIXED_PI)
    x += FIXED_2PI;

  if (x > FIXED_PIDIV4)
  {
    if (x < FIXED_PIDIV2+FIXED_PIDIV4)
    {
      return fpcos(x - FIXED_PIDIV2);
    }
    else
    {
      return fpsin(FIXED_PI - x);
    }
  }

  if (x < (-FIXED_PIDIV4))
  {
    if (x > (-FIXED_PIDIV2 - FIXED_PIDIV4))
    {
      return -fpcos(x + FIXED_PIDIV2);
    }
    else
    {
      return fpsin(-FIXED_PI - x);
    }
  }

  x2 = fpmul(x, x);

  // Pade approximation for sin(x)
  // x(60-7x)/(60+3x)
  numerator = fpmul(x, INT_TO_FIXED(60) - fpmul(INT_TO_FIXED(7), x2));

  denominator = INT_TO_FIXED(60) + fpmul(INT_TO_FIXED(3), x2);

  return fpdiv(numerator, denominator);
}

int32 fpcos(int32 x)
{
  register int32 x2;
  register int32 numerator;
  register int32 denominator;

  while (x > FIXED_PI)
    x -= FIXED_2PI;

  while (x < -FIXED_PI)
    x += FIXED_2PI;

  if (x > FIXED_PIDIV4)
  {
    if (x < FIXED_PIDIV2+FIXED_PIDIV4)
    {
      return -fpsin(x - FIXED_PIDIV2);
    }
    else
    {
      return -fpcos(FIXED_PI - x);
    }
  }

  if (x < (-FIXED_PIDIV4))
  {
    if (x > (-FIXED_PIDIV2 - FIXED_PIDIV4))
    {
      return fpsin(x + FIXED_PIDIV2);
    }
    else
    {
      return -fpcos(-FIXED_PI - x);
    }
  }

  x2 = fpmul(x, x);

  // Pade approximation for cos(x)
  // (12-5x)/(12+x)

  numerator = INT_TO_FIXED(12) - fpmul(INT_TO_FIXED(5), x2);

  denominator = INT_TO_FIXED(12) + x2;

  return fpdiv(numerator, denominator);
}

int32 fptan(int32 x)
{
  register int32 x2;
  register int32 numerator;
  register int32 denominator;

  while (x > FIXED_PI)
    x -= FIXED_2PI;

  while (x < -FIXED_PI)
    x += FIXED_2PI;

  if (x > FIXED_PIDIV4)
  {
    if (x < FIXED_PIDIV2+FIXED_PIDIV4)
    {
      if ((x >= FIXED_PIDIV2 - 2) &&
          (x <= FIXED_PIDIV2 + 2))
        return ((x <= FIXED_PIDIV2) ? 0x7FFFFFFF : 0x80000000);

      return fpdiv(FIXED_ONE, fptan(FIXED_PIDIV2 - x));
    }
    else
    {
      return -fptan(FIXED_PI - x);
    }
  }

  if (x < (-FIXED_PIDIV4))
  {
    if (x > (-FIXED_PIDIV2 - FIXED_PIDIV4))
    {
      if ((x >= -FIXED_PIDIV2 - 2) &&
          (x <= -FIXED_PIDIV2 + 2))
        return ((x <= -FIXED_PIDIV2) ? 0x7FFFFFFF : 0x80000000);

      return fpdiv(FIXED_ONE, fptan(-FIXED_PIDIV2 - x));
    }
    else
    {
      return -fptan(-FIXED_PI - x);
    }
  }

  x2 = fpmul(x, x);

  // Pade approximation for tan(x)
  // x(15-x)/(15-6x)

  numerator = fpmul(x, INT_TO_FIXED(15) - x2);

  denominator = INT_TO_FIXED(15) - fpmul(INT_TO_FIXED(6), x2);

  return fpdiv(numerator, denominator);
}

int32 fpatan(int32 x)
{
  register int32 x2;
  register int32 numerator;
  register int32 denominator;

  if (x > FIXED_ONE)
  {
    return FIXED_PIDIV2 - fpatan(fpdiv(FIXED_ONE, x));
  }

  if (x < -FIXED_ONE)
  {
    return -FIXED_PIDIV2 - fpatan(fpdiv(FIXED_ONE, x));
  }

  x2 = fpmul(x, x);

  // Pade approximation for atan(x)
  // x(15+4x)/(15+9x)

  numerator = fpmul(x, INT_TO_FIXED(15) + fpmul(INT_TO_FIXED(4), x2));

  denominator = INT_TO_FIXED(15) + fpmul(INT_TO_FIXED(9), x2);

  return fpdiv(numerator, denominator);
}

int32 fpexp(int32 x)
{
  register int32 x_div_ln2;
  register int32 int_x;
  register int32 frac_x;
  register int32 int_exp;
  register int32 frac_exp;

  if (x > 0)
  {
    x_div_ln2 = fpdiv(x, FIXED_LN2);
    int_x = x_div_ln2 >> 16;
    frac_x = x_div_ln2 & 0x0000FFFF;

    int_exp = FIXED_ONE;
    while (int_x)
    {
      int_exp <<= 1;
      int_x--;
    }

    // Pade approximation for exp2(x)
    // exp2(x) = (3 + x)/(3 - x)
    // accurate to within 1% over [0, 1]

    frac_exp = fpdiv((INT_TO_FIXED(3) + frac_x),
                     (INT_TO_FIXED(3) - frac_x));

    return fpmul(int_exp, frac_exp);
  }
  else if (x < 0)
  {
    return fpdiv(FIXED_ONE, fpexp(-x));
  }
  else
  {
    return FIXED_ONE;
  }
}

int32 fpln(int32 x)
{
  register int32 numerator;
  register int32 denominator;
  register int32 a = 0;

  // if x = y * 2^a then
  // ln(x) = ln(y * 2^a)
  // ln(x) = ln(y) + ln(2^a)
  // ln(x) = ln(y) + a * ln(2)

  if (x <= 0)
  {
    return 0;
  }
  else if (x > FIXED_ONE)
  {
    while (x & 0xFFFF0000)
    {
      x >>= 1;
      a++;
    }
  }
  else if (x < FIXED_ONE)
  {
    while (!(x & 0xFFFF0000))
    {
      x <<= 1;
      a--;
    }

    x >>= 1;
    a++;
  }

  // Pade approximation for ln(x+1)
  // ln(x+1) = x(6+x)/(6+4x)

  x -= FIXED_ONE;

  numerator = fpmul(x, INT_TO_FIXED(6) + x);

  denominator = INT_TO_FIXED(6) + fpmul(INT_TO_FIXED(4), x);

  return fpdiv(numerator, denominator) + fpmul(INT_TO_FIXED(a), FIXED_LN2);
}
