/*
 * $Id: exposure.c,v 1.1 2001/12/30 03:26:22 dwalters Exp $
 *
 * Auto-exposure algorithm for gbcam library.
 *
 * Copyright (C) 2001 Dafydd Walters
 * dwalters@users.sourceforge.net
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

#include <68332/mrm/gbcam.h>

#define ABS(x) ((x) > 0 ? (x) : -(x))

static int mean_deviation(void);

/*
 * Sets the exposure time automatically by metering
 * the image brightness several times using a binary-search
 * algorithm to maximize the mean deviation of the pixel
 * intensity values. Call this before calling gbcam_shoot().
 * If you need to adjust any of the M64282FP's registers,
 * change the members of gbcam_registers prior to calling
 * this function. This function overwrites the value of
 * member C in the gbcam_registers structure.
 */
void gbcam_exposure(void)
{
    uint16 min_exposure = 0;
    uint16 max_exposure = 0xffff;
    uint16 upper_attempt;
    uint16 lower_attempt;
    uint16 center_exposure;
    int md_upper = 255, md_lower = 0;

    while ((ABS(min_exposure - max_exposure) > 32) &&
            ((ABS(md_upper - md_lower) > 5) || (md_upper < 10)))
    {
        center_exposure = (min_exposure + max_exposure) / 2;
        upper_attempt  = (max_exposure + center_exposure) / 2;
        lower_attempt = (min_exposure + center_exposure) / 2;

        gbcam_registers.C = upper_attempt;
        gbcam_shoot();
        md_upper = mean_deviation();

        gbcam_registers.C = lower_attempt;
        gbcam_shoot();
        md_lower = mean_deviation();

        if (md_upper > md_lower)
            min_exposure = center_exposure;
        else
            max_exposure = center_exposure;
    }

    gbcam_registers.C = center_exposure;
}

int mean_deviation(void)
{
    long total = 0;
    long tot_diffs = 0;
    int mean;
    int i;

    for (i = 0; i < gbcam_IMAGE_SIZE; i++)
        total += gbcam_imageBuffer[i];

    mean = (int)(total / gbcam_IMAGE_SIZE);

    for (i = 0; i < gbcam_IMAGE_SIZE; i++)
        tot_diffs += (ABS((int)gbcam_imageBuffer[i] - mean));

    return tot_diffs / gbcam_IMAGE_SIZE;
}
