/*
 * $Id: sim.c,v 1.5 2003/12/07 07:29:01 dhylands Exp $
 *
 * Copyright (C) 2001 Doug Chapman 
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

#include "sim_regs.h"
#include <68332/sim.h>
#include <68332/isr.h>
#include <stdio.h>

SIM_T *SIM = (SIM_T *)SIM_BASE;

/*
 * set processor speed
 *
 * Set the processor speed.
 * The upper 8 bits of the argument are placed into the
 * upper 8 bits of the clock synthesizer control register.  This
 * will not return until the SLOCK bit shows the synthesizer is
 * locked in on this speed.
 *
 * Returns 0 on success, non 0 on failure.
 */
int set_processor_speed(int speed){
    uint16 tmp;

    tmp = SIM->SYNCR.FULLWORD;
    tmp = tmp & 0x00ff;
    tmp = tmp | speed;
    SIM->SYNCR.FULLWORD = tmp;

    while (SIM->SYNCR.BITS.SLOCK == 0) ;
    // while ((*(short int *) (0xfffa04) & 0x8) == 0);

    return 0;
}

/*
 * get processor frequency
 *
 * Retrieves the processor speed that was set.
 */
uint32 get_processor_freq(void){

    /*
     * The SIM Documentation gives the formula for calculation Fsystem as:
     *
     * Freference[4( Y + 1 )(2^(2W + X))]
     *
     * 2W + X is the same thing as SIM->SYNCR.FULLWORD >> 14
     * multiplying by 2^(2W + X) is the same thing as shifting left by 2W+X
     *
     * For example, 25 MHz sets SYNCR to D700 (1101011100000000) has
     * 2W+X = 3, Y = 010111 = 23.
     *
     * 32768 * 4 * 24 << 3 = 25,165,824
     */

    return (FCRYSTAL * 4 * ( SIM->SYNCR.BITS.Y + 1))
        << ((SIM->SYNCR.FULLWORD >> 14) & 0x3);
}

/*
 * get_reset_state
 *
 * Return one of the enumerated types above.  Return negative
 * value on error.
 */
reset_state_t get_reset_state(void){

    if (SIM->RSR.BITS.EXT)
	return RESET_EXT;

    if (SIM->RSR.BITS.POW)
	return RESET_POW;

    if (SIM->RSR.BITS.SW)
	return RESET_SW;

    if (SIM->RSR.BITS.HLT)
	return RESET_HLT;

    if (SIM->RSR.BITS.SYS)
	return RESET_SYS;

    if (SIM->RSR.BITS.TST)
	return RESET_TST;

    return 0;
}


/*
 * read_porte
 *
 * Return the current 8 bit value of the porte data register
 */
uint8 read_porte(void){

    return SIM->PORTE1.BITS.DATA;
}

/*
 * write_porte
 *
 * Writes a value to the porte data register
 */

void write_porte( uint8 val )
{
    SIM->PORTE1.BITS.DATA = val;
}

/*
 * set_porte_data_direction
 *
 * Set the pins on port E to input or output.
 * Argument is an 8 bit value with each bit corresponding to
 * an port E pin.
 * Setting a bit to 1 makes the pin an output.
 * Setting a bit to 0 makes the pin an input.
 *
 * Returns 0 on success, non 0 on error.
 */
int set_porte_data_direction(uint8 direction_mask){
    SIM->DDRE.BITS.DATA_DIR = direction_mask;

    return 0;
}

/*
 * set_porte_pin_assignment
 *
 * Sets pins in port E to either standard port E I/O or a control
 * bus signal.  See section D.2.8 of the 68332 users guide for details.
 * Argument is an 8 bit value with each bit corresponding to a pin
 * on port E.
 *
 * Setting a bit to 1 sets it to a control signal (as in section D.2.8)
 * Setting a bit to 0 sets it to a port E pin.
 *
 * Returns 0 on success, non 0 on error.
 */
int set_porte_pin_assignment(uint8 pin_assignment){
    SIM->PEPAR.BITS.PIN_ASSIGN = pin_assignment;

    return 0;
}

/*
 * read_portf
 *
 * Return the current 8 bit value of the portf data register
 */
uint8 read_portf(void){
    return SIM->PORTF0.BITS.DATA;
}

/*
 * write_porte
 *
 * Writes a value to the porte data register
 */

void write_portf( uint8 val )
{
    SIM->PORTF0.BITS.DATA = val;
}

/*
 * set_portf_data_direction
 *
 * Set the pins on port F to input or output.
 * Argument is an 8 bit value with each bit corresponding to
 * a port F pin.
 * Setting a bit to 1 makes the pin an output.
 * Setting a bit to 0 makes the pin an input.
 *
 * Returns 0 on success, non 0 on error.
 */
int set_portf_data_direction(uint8 direction_mask){
    SIM->DDRF.BITS.DATA_DIR = direction_mask;

    return 0;
}

/*
 * set_portf_pin_assignment
 *
 * Sets pins in port F to either standard port E I/O or a control
 * bus signal.  See section D.2.11 of the 68332 users guide for details.
 * Argument is an 8 bit value with each bit corresponding to a pin
 * on port F.
 *
 * Setting a bit to 1 sets it to a control signal (as in section D.2.11)
 * Setting a bit to 0 sets it to a port E pin.
 *
 * Returns 0 on success, non 0 on error.
 */
int set_portf_pin_assignment(uint8 pin_assignment){
    SIM->PFPAR.BITS.PIN_ASSIGN = pin_assignment;

    return 0;
}


/*
 * set_watchdog
 *
 * Turns on/off the system watchdog.  This can be used to reset the
 * system if the watchdog timer is not reset (see reset_watchdog).
 *
 * Arguments:
 *	enable:
 *		0 = watchdog disable
 *		1 = watchdog enable
 *
 *	prescale:
 *		0 = watchdog clock not prescaled
 *		1 = watchdog clock prescaled by 512
 *
 *	timeout:
 *		0 = watchdog timeout after 2^9 ticks
 *		1 = watchdog timeout after 2^11 ticks
 *		2 = watchdog timeout after 2^13 ticks
 *		3 = watchdog timeout after 2^15 ticks
 *
 * Returns 0 on success, non 0 on error.
 */
int set_watchdog(int enable, int prescale, int timeout){
    if (enable < 0 || enable > 1)
	return -1;
    if (prescale < 0 || prescale > 1)
	return -1;
    if (timeout < 0 || timeout > 3)
	return -1;

    SIM->SYPCR.BITS.SWE = enable;
    SIM->SYPCR.BITS.SWP = prescale;
    SIM->SYPCR.BITS.SWT = timeout;

    return 0;
}

/*
 * reset_watchdog
 *
 * Must be called before the watchdog timer is hit to prevent watchdog reset
 *
 */
void reset_watchdog(void){

    SIM->SWSR.BITS.RESET = 0x55;
    SIM->SWSR.BITS.RESET = 0xaa;

    return;
}

/*
 * periodic_interrupt_setup
 *
 * Set up a periodic interrupt.
 *
 * Arguments:
 *	function - C function to be called on interrupt
 *	level - interrupt request level
 *      vector - interrupt request vector
 *	prescale:
 *		0 = periodic timer not prescaled
 *		1 = periodic timer prescaled by 512
 *
 *	timer - clock ticks to run before interrupt is triggered (8 bits)
 *
 * Returns 0 on success, non 0 on failure
 */
int periodic_interrupt_setup(void (*function)(),
			     int level, int vector,
			     int prescale, uint8 timer)
{
  install_isr(vector, function);

  SIM->PITR.BITS.PTP = prescale;
  SIM->PITR.BITS.PITM = timer;
  SIM->PICR.BITS.PIRQL = level;
  SIM->PICR.BITS.PIV = vector;

  return 0;
}

/*
 * The following routines exists as helpers for some of the I/O routines.
 * It allows a memory read to NOT be optimized out by the compiler.
 *
 * For example, it is perfectly legal for the compiler to reorder:
 *
 *    val1 = *some_mem_addr;
 *    *some_memaddr2 = val2;
 *
 * into this:
 *
 *    *some_memaddr2 = val2;
 *    val1 = *some_mem_addr;
 *
 * Both of the above are functionally identical except when the memory
 * addresses point at some type of peripheral device that relies on the
 * accesses being in a particular order.
 *
 * The compiler will NOT move instructions like these across function calls.
 * Having this routine do nothing takes a couple of CPU cycles but prevents
 * the compiler from moving the order of things around.
 */

void barrier_read_uint16( uint16 val )
{
    (void)val;
}

void barrier( void )
{
    ;
}
