/*
 * $Id: tpu.c,v 1.9 2003/12/07 07:23:52 dhylands Exp $
 *
 * This file contains the lowlevel
 * functions used to access the TPU.
 *
 * Copyright (C) 2001 Mike Panetta
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

#include <stdio.h>

#include <68332/tpu.h>
#include "tpu_regs.h"
#include <68332/types.h>
#include <68332/sim.h>

// #define DEBUG 1 /* This needs to be done elsewhere */

#ifdef DEBUG
#define SET_ERRNO(objhandle, value) \
        do { objhandle->errno = value; } while (0)
#define DPRINTF(fmt, args...) \
        do { iprintf(fmt, ##args); } while (0)
#else
#define SET_ERRNO(objhandle, value) \
        do { } while (0)
#define DPRINTF(fmt, args...) \
        do { } while (0)
#endif

/* The global TPU object(s). */
static TPUObject _tpu[NUM_TPUS];
static int tpu_base[MAX_TPUS] = { TPU0_BASE, TPU1_BASE };
// Instead of static allocation, the below could also be
// malloc'd in tpu_request_channel().
static TPUChannel _tpuChannel[NUM_TPUS][CHANS_PER_TPU];

int
tpu_init(uint16 tpu_num)
{
  return tpu_init_extended(tpu_num, 0, 0, 1);
}

int
tpu_init_extended(uint16 tpu_num, uint16 tcr1p, uint16 tcr2p, uint16 psck)
{
  TPU_T   * tpu;
  TPUMCR_T  tpuMCR;
  uint32    cpuFreq;

  if (tpu_num >= NUM_TPUS)
  {
    return -1;
  }

  /* Set up the TPUObject structure */
  _tpu[tpu_num].TPUHardware   = (void *)tpu_base[tpu_num];
  _tpu[tpu_num].channelsInUse = 0;
  _tpu[tpu_num].initialized   = 0;

  tpu = ((TPU_T *) _tpu[tpu_num].TPUHardware);

  tpuMCR.FULLWORD = tpu->TPUMCR.FULLWORD;
  tpuMCR.BITS.STOP  = 0;
  tpuMCR.BITS.TCR1P = tcr1p;
  tpuMCR.BITS.TCR2P = tcr2p;
  tpuMCR.BITS.EMU   = 0;
  tpuMCR.BITS.SUPV  = 1;
  tpuMCR.BITS.PSCK  = psck;
  tpu->TPUMCR.FULLWORD = tpuMCR.FULLWORD;

  /*
   * Since all of the prescalar's are powers of 2, we can get away with
   * just doing a shift.
   */
  
  cpuFreq = get_processor_freq();

  _tpu[tpu_num].tcr1Freq = cpuFreq >> ( tcr1p + ( psck ? 2 : 5 ));
  _tpu[tpu_num].tcr2Freq = cpuFreq >> ( tcr2p + 3 );

  _tpu[tpu_num].initialized = 1;

  return 0;
}

uint32 tpu_get_tcr1_freq( uint16 tpu_num )
{
    if ( tpu_num < NUM_TPUS )
    {
        return _tpu[ tpu_num ].tcr1Freq;
    }
    return 0;
}

uint32 tpu_get_tcr2_freq( uint16 tpu_num )
{
    if ( tpu_num < NUM_TPUS )
    {
        return _tpu[ tpu_num ].tcr2Freq;
    }
    return 0;
}

int
tpu_init_interrupts(uint16 tpu_num, uint16 iarb, uint16 cirl, uint16 cibv)
{
  TPU_T * tpu;
  TPUMCR_T tpuMCR;
  TPUTICR_T tpuTICR;

  if (tpu_num >= NUM_TPUS)
  {
    return -1;
  }

  if (!_tpu[tpu_num].initialized)
  {
    return -1;
  }

  tpu = ((TPU_T *) _tpu[tpu_num].TPUHardware);

  tpuMCR.FULLWORD = tpu->TPUMCR.FULLWORD;
  tpuMCR.BITS.IARB  = iarb;
  tpu->TPUMCR.FULLWORD = tpuMCR.FULLWORD;

  tpuTICR.FULLWORD = tpu->TICR.FULLWORD;
  tpuTICR.BITS.CIRL  = cirl;
  tpuTICR.BITS.CIBV  = cibv;
  tpu->TICR.FULLWORD = tpuTICR.FULLWORD;

  return 0;
}

TPUChannel*
tpu_request_channel(uint16 tpu_num, uint16 channel_num)
{
  if (tpu_num >= NUM_TPUS)
  {
    return NULL;
  }

  if (channel_num >= CHANS_PER_TPU)
  {
    return NULL;
  }

  if (!_tpu[tpu_num].initialized)
  {
    return NULL;
  }

  DPRINTF("tpu_request_channel() : channelsInUse = 0x%x\n", _tpu[tpu_num].channelsInUse);

  if ((_tpu[tpu_num].channelsInUse >> channel_num) & 0x1)
  {
    /* Channel is already reserved */
    return NULL;
  }

  _tpu[tpu_num].channelsInUse = _tpu[tpu_num].channelsInUse | (1 << channel_num);

  DPRINTF("tpu_request_channel() : channelsInUse = 0x%04x\n", _tpu[tpu_num].channelsInUse);

  _tpuChannel[tpu_num][channel_num].channel_num = channel_num;
  _tpuChannel[tpu_num][channel_num].initialized = 0;
  _tpuChannel[tpu_num][channel_num].TPUHardware = _tpu[tpu_num].TPUHardware;
  _tpuChannel[tpu_num][channel_num].errno = 0;
  _tpuChannel[tpu_num][channel_num].master = NULL;
  _tpuChannel[tpu_num][channel_num].userData = NULL;

  return &_tpuChannel[tpu_num][channel_num];
}

int
tpu_release_channel(uint16 tpu_num, uint16 channel_num)
{
  if (tpu_num >= NUM_TPUS)
  {
    return -1;
  }

  DPRINTF("tpu_release_channel() : channelsInUse = 0x%04x\n", _tpu[tpu_num].channelsInUse);

  _tpu[tpu_num].channelsInUse =
    _tpu[tpu_num].channelsInUse & (~(1 << channel_num));

  DPRINTF("tpu_release_channel() : channelsInUse = 0x%04x\n", _tpu[tpu_num].channelsInUse);

  return 0;
}

int
tpu_set_CIER(TPUChannel* handle, uint16 value)
{
  TPU_T * tpu;
  
  if (handle == NULL)
  {
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  tpu->CIER.FULLWORD &= (~(0x1<<handle->channel_num));
  tpu->CIER.FULLWORD |= ((0x1<<handle->channel_num) & (value << handle->channel_num));

  DPRINTF("tpu_set_CIER() : CIER = 0x%04x\n", tpu->CIER.FULLWORD);
	
  return 0;
}

int
tpu_get_CISR(TPUChannel* handle)
{
  TPU_T * tpu;
  uint16 value;
  
  if (handle == NULL)
  {
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  DPRINTF("tpu_get_CISR() : CISR = 0x%04x\n", tpu->CISR.FULLWORD);

  value = (tpu->CISR.FULLWORD >> handle->channel_num) & 0x1;

  return value;
}

int
tpu_clear_CISR(TPUChannel* handle)
{
  TPU_T * tpu;
  
  if (handle == NULL)
  {
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  DPRINTF("tpu_clear_CISR() : CISR = 0x%04x\n", 
	  tpu->CISR.FULLWORD);

  /*
   * The documentation in the TPU RM specifies that to clear a status flag
   * we must first read the CISR and then write a zero to the appropriate bit.
   */

  barrier_read_uint16( tpu->CISR.FULLWORD );

  tpu->CISR.FULLWORD = (~(0x1<<handle->channel_num));

  DPRINTF("tpu_clear_CISR() : CISR = 0x%04x\n", 
	  tpu->CISR.FULLWORD);

  return 0;
}

int
tpu_set_CFSR(TPUChannel* handle, uint16 function_code)
{
  TPU_T * tpu;
  uint16 regnum = 0;
  uint16 regpart = 0;
  
  if (handle == NULL)
  {
    return -1;
  }

  if (function_code > 15)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  regnum = 3 - (handle->channel_num/4);
  regpart = (handle->channel_num % 4) * 4;

  DPRINTF("tpu_set_CFSR() : regnum = %d, regpart = %d\n",
	  regnum, regpart);

  DPRINTF("tpu_set_CFSR() : CFSR = 0x%04x (%p)\n", 
	  tpu->CFSR.CFSR[regnum], &tpu->CFSR.CFSR[regnum]);

  tpu->CFSR.CFSR[regnum] &= (~(0xF << regpart));
  tpu->CFSR.CFSR[regnum] |= ((0xF << regpart) & (function_code << regpart));

  DPRINTF("tpu_set_CFSR() : CFSR = 0x%04x\n", 
	  tpu->CFSR.CFSR[regnum]);

  return 0;
}

int
tpu_get_CFSR(TPUChannel* handle)
{
  TPU_T * tpu;
  uint16 function_code;
  uint16 regnum = 0;
  uint16 regpart = 0;
  
  if (handle == NULL)
  {
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  regnum = 3 - (handle->channel_num/4);
  regpart = (handle->channel_num % 4) * 4;

  function_code = (tpu->CFSR.CFSR[regnum] >> regpart) & 0xF;

  return function_code;
}

int
tpu_set_HSQR(TPUChannel* handle, uint16 value)
{
  TPU_T * tpu;
  uint16 regnum = 0;
  uint16 regpart = 0;
  
  if (handle == NULL)
  {
    return -1;
  }

  if (value > 3)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  regnum = 1 - (handle->channel_num/8);
  regpart = (handle->channel_num % 8) * 2;

  DPRINTF("tpu_set_HSQR() : regnum = %d, regpart = %d\n",
	  regnum, regpart);

  DPRINTF("tpu_set_HSQR() : HSQR = 0x%04x\n", 
	  tpu->HSQR.HSQR[regnum]);

  tpu->HSQR.HSQR[regnum] &= (~(0x3 << regpart));
  tpu->HSQR.HSQR[regnum] |= ((0x3 << regpart) & (value << regpart));

  DPRINTF("tpu_set_HSQR() : HSQR = 0x%04x\n", 
	  tpu->HSQR.HSQR[regnum]);

  return 0;
}

int
tpu_get_HSQR(TPUChannel* handle)
{
  TPU_T * tpu;
  uint16 value;
  uint16 regnum = 0;
  uint16 regpart = 0;
 
  if (handle == NULL)
  {
    return -1; 
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  regnum = 1 - (handle->channel_num/8);
  regpart = (handle->channel_num % 8) * 2;

  value = (tpu->HSQR.HSQR[regnum] >> regpart) & 0x3;

  return value;
}

int
tpu_set_HSRR(TPUChannel* handle, uint16 value)
{
  TPU_T * tpu;
  uint16 regnum = 0;
  uint16 regpart = 0;
  
  if (handle == NULL)
  {
    return -1;
  }

  if (value > 3)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  regnum = 1 - (handle->channel_num/8);
  regpart = (handle->channel_num % 8) * 2;
	
  DPRINTF("tpu_set_HSRR() : regnum = %d, regpart = %d\n",
	  regnum, regpart);

  DPRINTF("tpu_set_HSRR() : HSRR = 0x%04x\n", 
	  tpu->HSRR.HSRR[regnum]);

  tpu->HSRR.HSRR[regnum] &= (~(0x3 << regpart));
  tpu->HSRR.HSRR[regnum] |= ((0x3 << regpart) & (value << regpart));

  DPRINTF("tpu_set_HSRR() : HSRR = 0x%04x\n", 
	  tpu->HSRR.HSRR[regnum]);

  return 0;
}

int
tpu_get_HSRR(TPUChannel* handle)
{
  TPU_T * tpu;
  uint16 value;
  uint16 regnum = 0;
  uint16 regpart = 0;

  if (handle == NULL)
  {
    return -1;
  }
  
  tpu = ((TPU_T *) handle->TPUHardware);

  regnum = 1 - (handle->channel_num/8);
  regpart = (handle->channel_num % 8) * 2;

  value = (tpu->HSRR.HSRR[regnum] >> regpart) & 0x3;

  return value;
}

int
tpu_set_CPR(TPUChannel* handle, uint16 value)
{
  TPU_T * tpu;
  uint16 regnum = 0;
  uint16 regpart = 0;
  
  if (handle == NULL)
  {
    return -1;
  }

  if (value > 3)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  regnum = 1 - (handle->channel_num/8);
  regpart = (handle->channel_num % 8) * 2;

  DPRINTF("tpu_set_CPR() : regnum = %d, regpart = %d\n",
	  regnum, regpart);

  DPRINTF("tpu_set_CPR() : CPR = 0x%04x\n", 
	  tpu->CPR.CPR[regnum]);

  tpu->CPR.CPR[regnum] &= (~(0x3 << regpart));
  tpu->CPR.CPR[regnum] |= ((0x3 << regpart) & (value << regpart));

  DPRINTF("tpu_set_CPR() : CPR = 0x%04x\n", 
	  tpu->CPR.CPR[regnum]);

  return 0;
}

int
tpu_get_CPR(TPUChannel* handle)
{
  TPU_T * tpu;
  uint16 value;
  uint16 regnum = 0;
  uint16 regpart = 0;

  if (handle == NULL)
  {
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  regnum = 1 - (handle->channel_num/8);
  regpart = (handle->channel_num % 8) * 2;

  value = (tpu->CPR.CPR[regnum] >> regpart) & 0x3;

  return value;
}

int
tpu_write_parameter_word(TPUChannel* handle, tpu_pram_word_t entry, uint16 value)
{
  TPU_T * tpu;

  if (handle == NULL)
  {
    return -1;
  }

  if (entry > 0xF)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  tpu->PRAM[handle->channel_num].PRAM[entry] = value;

  return 0;
}

int
tpu_write_parameter_byte(TPUChannel* handle, tpu_pram_byte_t entry, uint8 value)
{
  TPU_T * tpu;

  if (handle == NULL)
  {
    return -1;
  }

  if (entry > 0xF)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  tpu->PRAM[handle->channel_num].PRAMBYTE[entry] = value;

  return 0;
}

int
tpu_read_parameter_word(TPUChannel* handle, tpu_pram_word_t entry, uint16 * value)
{
  TPU_T * tpu;

  if (handle == NULL)
  {
    return -1;
  }

  if (entry > 0xF)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  if (value == NULL)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  *value = tpu->PRAM[handle->channel_num].PRAM[entry];

  return 0;
}

int
tpu_read_parameter_byte(TPUChannel* handle, tpu_pram_byte_t entry, uint8 * value)
{
  TPU_T * tpu;

  if (handle == NULL)
  {
    return -1;
  }

  if (entry > 0xF)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  if (value == NULL)
  {
    SET_ERRNO(handle, TPU_ARGUMENT_OUT_OF_RANGE);
    return -1;
  }

  tpu = ((TPU_T *) handle->TPUHardware);

  *value = tpu->PRAM[handle->channel_num].PRAMBYTE[entry];

  return 0;
}

#ifdef DEBUG

typedef struct
{
  tpu_errno_t errno;
  char * errmsg;
} tpu_errmsg_t;

tpu_errmsg_t tpu_errmsg[] = {
				{ TPU_ERROR_NONE, "Success" },
				{ TPU_CHANNEL_IN_USE, "Channel in use" },
				{ TPU_CHANNEL_OUT_OF_RANGE, "Channel number out of range" },
				{ TPU_ARGUMENT_OUT_OF_RANGE, "Argument out of range" },
				{ 0, NULL }
};

void
tpu_perror(TPUChannel* handle)
{
  int i = 0;

  do {
    if (handle->errno == tpu_errmsg[i].errno)
    {
      iprintf("TPU Error: %s\n", tpu_errmsg[i].errmsg);
      return;
    }
    i++;
  } while (tpu_errmsg[i].errmsg != NULL);

  iprintf("Unknown error %d\n", handle->errno);

  return;
}

#else

void
tpu_perror(TPUChannel* handle)
{
  /* nothing */
}

#endif

// vim: ts=4
// vim: sw=4
