/*
 * $Id: tpu_regs.h,v 1.5 2002/10/03 01:22:17 dwalters Exp $
 *
 * Register definitions for the 68332 time processor unit.
 *
 * Copyright (C) 2001 Mike Panetta
 *
 * This library is distributed under the terms of the GNU Lesser General
 * Public License (LGPL).  You should have received a copy of the
 * license along with this library (LGPL.txt); if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

#ifndef __TPU_REGS_H__
#define __TPU_REGS_H__

#include <68332/types.h>

/*
 * Note, all register definitions use the following convention:
 *
 *	typedef union {
 *		struct {
 *			uint16 <bit group name> : <number of bits>
 *			uint16 <bit group name> : <number of bits>
 *			...
 *			uint16 <bit group name> : <number of bits>
 *			(note, total bits should = 8/16 for byte/word)
 *		} BITS;
 *		uint16 FULLWORD;
 *	} name_t;
 *
 * This allows access to certain bits within a word as a structure
 * member.  This limits the need for bitmasks and bit shifts.  The
 * union allows the full word (FULLWORD) to be accessed as well.
 */

#define CHANS_PER_TPU 16
#define NUM_TPUS 1
#define MAX_TPUS 2
#define TPU0_BASE (0xFFFE00L)
// The following define has an arbitrary value for now.
#define TPU1_BASE (0L)


typedef union {
	struct {
		uint16 STOP:1;		/* Used to shut down the TPU module. */
		uint16 TCR1P:2;		/* Timer Count Register 1 Prescaler Control */
		uint16 TCR2P:2;		/* Timer Count Register 2 Prescaler Control */
		uint16 EMU:1;		/* Enable Emulation Mode. */
		uint16 T2CG:1;		/* TCR2 Clock/Gate Control (TPU2 Only)*/
		uint16 STF:1;		/* Stop Flag */
		uint16 SUPV:1;		/* Supervisor access mode */
		uint16 PSCK:1;		/* Prescaler Clock */
		uint16 TPU2:1;		/* TPU2 Enable (TPU2 Only) */
		uint16 T2SCL:1;		/* TCR2 Counter Edge Clock (TPU2 Only) */
		uint16 IARB:4;		/* Interrupt Arbitration Number */
	} BITS;
	uint16 FULLWORD;
} TPUMCR_T;

typedef union {
	struct {
		uint16 unused:16; 	/* Test Use Only */
	} BITS;
	uint16 FULLWORD;
} TPUTCR_T;

typedef union {
	struct {
		uint16 unused:16; 	/* Test Use Only */
	} BITS;
	uint16 FULLWORD;
} TPUDSCR_T;

typedef union {
	struct {
		uint16 unused:16; 	/* Test Use Only */
	} BITS;
	uint16 FULLWORD;
} TPUDSSR_T;

typedef union {
	struct {
		uint16 UNUSED0:5; 	/* Unused */
		uint16 CIRL:3;		/* Channel Interrupt Request Level */
		uint16 CIBV:4;		/* Channel Interrupt Base Vector */
		uint16 UNUSED1:4;	/* Unused */
	} BITS;
	uint16 FULLWORD;
} TPUTICR_T;

typedef union {
	struct {
		uint16 CH15:1; 	/* Channel Interrupt Enable/Disable Channel 15 */
		uint16 CH14:1; 	/* Channel Interrupt Enable/Disable Channel 14 */
		uint16 CH13:1; 	/* Channel Interrupt Enable/Disable Channel 13 */
		uint16 CH12:1; 	/* Channel Interrupt Enable/Disable Channel 12 */
		uint16 CH11:1; 	/* Channel Interrupt Enable/Disable Channel 11 */
		uint16 CH10:1; 	/* Channel Interrupt Enable/Disable Channel 10 */
		uint16 CH09:1; 	/* Channel Interrupt Enable/Disable Channel 09 */
		uint16 CH08:1; 	/* Channel Interrupt Enable/Disable Channel 08 */
		uint16 CH07:1; 	/* Channel Interrupt Enable/Disable Channel 07 */
		uint16 CH06:1; 	/* Channel Interrupt Enable/Disable Channel 06 */
		uint16 CH05:1; 	/* Channel Interrupt Enable/Disable Channel 05 */
		uint16 CH04:1; 	/* Channel Interrupt Enable/Disable Channel 04 */
		uint16 CH03:1; 	/* Channel Interrupt Enable/Disable Channel 03 */
		uint16 CH02:1; 	/* Channel Interrupt Enable/Disable Channel 02 */
		uint16 CH01:1; 	/* Channel Interrupt Enable/Disable Channel 01 */
		uint16 CH00:1; 	/* Channel Interrupt Enable/Disable Channel 00 */
	} BITS;
	uint16 FULLWORD;
} TPUCIER_T;

typedef union {
	struct {
		struct {
			uint16 CH15:4;	/* Channel Function Select */
			uint16 CH14:4;	/* Channel Function Select */
			uint16 CH13:4;	/* Channel Function Select */
			uint16 CH12:4;	/* Channel Function Select */
		} CFSR0;
		struct {
			uint16 CH11:4;	/* Channel Function Select */
			uint16 CH10:4;	/* Channel Function Select */
			uint16 CH09:4;	/* Channel Function Select */
			uint16 CH08:4;	/* Channel Function Select */
		} CFSR1;
		struct {
			uint16 CH07:4;	/* Channel Function Select */
			uint16 CH06:4;	/* Channel Function Select */
			uint16 CH05:4;	/* Channel Function Select */
			uint16 CH04:4;	/* Channel Function Select */
		} CFSR2;
		struct {
			uint16 CH03:4;	/* Channel Function Select */
			uint16 CH02:4;	/* Channel Function Select */
			uint16 CH01:4;	/* Channel Function Select */
			uint16 CH00:4;	/* Channel Function Select */
		} CFSR3;
	} I; /* For individual? */
	uint16 CFSR[4]; /* All of em together */
} TPUCFSR_T;

typedef union {
	struct {
		struct {
			uint16 CH15:2;	/* Host Sequence Register */
			uint16 CH14:2;	/* Host Sequence Register */
			uint16 CH13:2;	/* Host Sequence Register */
			uint16 CH12:2;	/* Host Sequence Register */
			uint16 CH11:2;	/* Host Sequence Register */
			uint16 CH10:2;	/* Host Sequence Register */
			uint16 CH09:2;	/* Host Sequence Register */
			uint16 CH08:2;	/* Host Sequence Register */
		} HSQR0;
		struct {
			uint16 CH07:2;	/* Host Sequence Register */
			uint16 CH06:2;	/* Host Sequence Register */
			uint16 CH05:2;	/* Host Sequence Register */
			uint16 CH04:2;	/* Host Sequence Register */
			uint16 CH03:2;	/* Host Sequence Register */
			uint16 CH02:2;	/* Host Sequence Register */
			uint16 CH01:2;	/* Host Sequence Register */
			uint16 CH00:2;	/* Host Sequence Register */
		} HSQR1;
	} I; /* For individual? */
	uint16 HSQR[2]; /* All of em together */
} TPUHSQR_T;

typedef union {
	struct {
		struct {
			uint16 CH15:2;	/* Host Service Request Register */
			uint16 CH14:2;	/* Host Service Request Register */
			uint16 CH13:2;	/* Host Service Request Register */
			uint16 CH12:2;	/* Host Service Request Register */
			uint16 CH11:2;	/* Host Service Request Register */
			uint16 CH10:2;	/* Host Service Request Register */
			uint16 CH09:2;	/* Host Service Request Register */
			uint16 CH08:2;	/* Host Service Request Register */
		} HSRR0;
		struct {
			uint16 CH07:2;	/* Host Service Request Register */
			uint16 CH06:2;	/* Host Service Request Register */
			uint16 CH05:2;	/* Host Service Request Register */
			uint16 CH04:2;	/* Host Service Request Register */
			uint16 CH03:2;	/* Host Service Request Register */
			uint16 CH02:2;	/* Host Service Request Register */
			uint16 CH01:2;	/* Host Service Request Register */
			uint16 CH00:2;	/* Host Service Request Register */
		} HSRR1;
	} I; /* For individual? */
	uint16 HSRR[2]; /* All of em together */
} TPUHSRR_T;

typedef union {
	struct {
		struct {
			uint16 CH15:2;	/* Channel Priority Register */
			uint16 CH14:2;	/* Channel Priority Register */
			uint16 CH13:2;	/* Channel Priority Register */
			uint16 CH12:2;	/* Channel Priority Register */
			uint16 CH11:2;	/* Channel Priority Register */
			uint16 CH10:2;	/* Channel Priority Register */
			uint16 CH09:2;	/* Channel Priority Register */
			uint16 CH08:2;	/* Channel Priority Register */
		} CPR0;
		struct {
			uint16 CH07:2;	/* Channel Priority Register */
			uint16 CH06:2;	/* Channel Priority Register */
			uint16 CH05:2;	/* Channel Priority Register */
			uint16 CH04:2;	/* Channel Priority Register */
			uint16 CH03:2;	/* Channel Priority Register */
			uint16 CH02:2;	/* Channel Priority Register */
			uint16 CH01:2;	/* Channel Priority Register */
			uint16 CH00:2;	/* Channel Priority Register */
		} CPR1;
	} I; /* For individual? */
	uint16 CPR[2]; /* All of em together */
} TPUCPR_T;

typedef union {
	struct {
		uint16 CH15:1; 	/* Channel Interrupt Status Channel 15 */
		uint16 CH14:1; 	/* Channel Interrupt Status Channel 14 */
		uint16 CH13:1; 	/* Channel Interrupt Status Channel 13 */
		uint16 CH12:1; 	/* Channel Interrupt Status Channel 12 */
		uint16 CH11:1; 	/* Channel Interrupt Status Channel 11 */
		uint16 CH10:1; 	/* Channel Interrupt Status Channel 10 */
		uint16 CH09:1; 	/* Channel Interrupt Status Channel 09 */
		uint16 CH08:1; 	/* Channel Interrupt Status Channel 08 */
		uint16 CH07:1; 	/* Channel Interrupt Status Channel 07 */
		uint16 CH06:1; 	/* Channel Interrupt Status Channel 06 */
		uint16 CH05:1; 	/* Channel Interrupt Status Channel 05 */
		uint16 CH04:1; 	/* Channel Interrupt Status Channel 04 */
		uint16 CH03:1; 	/* Channel Interrupt Status Channel 03 */
		uint16 CH02:1; 	/* Channel Interrupt Status Channel 02 */
		uint16 CH01:1; 	/* Channel Interrupt Status Channel 01 */
		uint16 CH00:1; 	/* Channel Interrupt Status Channel 00 */
	} BITS;
	uint16 FULLWORD;
} TPUCISR_T;

typedef union {
	struct {
		uint16 CH15:1; 	/* Link Register Channel 15 */
		uint16 CH14:1; 	/* Link Register Channel 14 */
		uint16 CH13:1; 	/* Link Register Channel 13 */
		uint16 CH12:1; 	/* Link Register Channel 12 */
		uint16 CH11:1; 	/* Link Register Channel 11 */
		uint16 CH10:1; 	/* Link Register Channel 10 */
		uint16 CH09:1; 	/* Link Register Channel 09 */
		uint16 CH08:1; 	/* Link Register Channel 08 */
		uint16 CH07:1; 	/* Link Register Channel 07 */
		uint16 CH06:1; 	/* Link Register Channel 06 */
		uint16 CH05:1; 	/* Link Register Channel 05 */
		uint16 CH04:1; 	/* Link Register Channel 04 */
		uint16 CH03:1; 	/* Link Register Channel 03 */
		uint16 CH02:1; 	/* Link Register Channel 02 */
		uint16 CH01:1; 	/* Link Register Channel 01 */
		uint16 CH00:1; 	/* Link Register Channel 00 */
	} BITS;
	uint16 FULLWORD;
} TPULR_T;

typedef union {
	struct {
		uint16 CH15:1; 	/* Service Grant Latch Register Channel 15 */
		uint16 CH14:1; 	/* Service Grant Latch Register Channel 14 */
		uint16 CH13:1; 	/* Service Grant Latch Register Channel 13 */
		uint16 CH12:1; 	/* Service Grant Latch Register Channel 12 */
		uint16 CH11:1; 	/* Service Grant Latch Register Channel 11 */
		uint16 CH10:1; 	/* Service Grant Latch Register Channel 10 */
		uint16 CH09:1; 	/* Service Grant Latch Register Channel 09 */
		uint16 CH08:1; 	/* Service Grant Latch Register Channel 08 */
		uint16 CH07:1; 	/* Service Grant Latch Register Channel 07 */
		uint16 CH06:1; 	/* Service Grant Latch Register Channel 06 */
		uint16 CH05:1; 	/* Service Grant Latch Register Channel 05 */
		uint16 CH04:1; 	/* Service Grant Latch Register Channel 04 */
		uint16 CH03:1; 	/* Service Grant Latch Register Channel 03 */
		uint16 CH02:1; 	/* Service Grant Latch Register Channel 02 */
		uint16 CH01:1; 	/* Service Grant Latch Register Channel 01 */
		uint16 CH00:1; 	/* Service Grant Latch Register Channel 00 */
	} BITS;
	uint16 FULLWORD;
} TPUSGLR_T;

typedef union {
	struct {
		uint16 CH15:1; 	/* Decoded Channel Number Register Channel 15 */
		uint16 CH14:1; 	/* Decoded Channel Number Register Channel 14 */
		uint16 CH13:1; 	/* Decoded Channel Number Register Channel 13 */
		uint16 CH12:1; 	/* Decoded Channel Number Register Channel 12 */
		uint16 CH11:1; 	/* Decoded Channel Number Register Channel 11 */
		uint16 CH10:1; 	/* Decoded Channel Number Register Channel 10 */
		uint16 CH09:1; 	/* Decoded Channel Number Register Channel 09 */
		uint16 CH08:1; 	/* Decoded Channel Number Register Channel 08 */
		uint16 CH07:1; 	/* Decoded Channel Number Register Channel 07 */
		uint16 CH06:1; 	/* Decoded Channel Number Register Channel 06 */
		uint16 CH05:1; 	/* Decoded Channel Number Register Channel 05 */
		uint16 CH04:1; 	/* Decoded Channel Number Register Channel 04 */
		uint16 CH03:1; 	/* Decoded Channel Number Register Channel 03 */
		uint16 CH02:1; 	/* Decoded Channel Number Register Channel 02 */
		uint16 CH01:1; 	/* Decoded Channel Number Register Channel 01 */
		uint16 CH00:1; 	/* Decoded Channel Number Register Channel 00 */
	} BITS;
	uint16 FULLWORD;
} TPUDCNR_T;

/* This register is only available on the TPU2 */
typedef union {
	struct {
		uint16 UNUSED:7;	/* Unused bits */
		uint16 DIV2:1;		/* Divide by 2 Control (TPU2 Only) */
		uint16 SOFTRST:1;	/* Soft Reset (TPU2 Only) */
		uint16 ETBANK:2;	/* Entry Table Bank Select (TPU2 Only) */
		uint16 FPSCK:3;		/* Filter Prescaler Clock Control (TPU2 Only) */
		uint16 T2CF:1;		/* T2CLK Pin Filter Control (TPU2 Only) */
		uint16 DTPU:1;		/* Disable TPU2 Pins (TPU2 Only) */
	} BITS;
	uint16 FULLWORD;
} TPUMCR2_T;

typedef union {
	struct {
		uint16 P00;
		uint16 P01;
		uint16 P02;
		uint16 P03;
		uint16 P04;
		uint16 P05;
		uint16 P06;
		uint16 P07;
	} PRAMN;
	uint16 PRAM[8];
	uint8  PRAMBYTE[16];
} TPUPRAM_T;

typedef struct {
	TPUMCR_T  TPUMCR;
	TPUTCR_T  TCR;
	TPUDSCR_T DSCR;
	TPUDSSR_T DSSR;
	TPUTICR_T TICR;
	TPUCIER_T CIER;
	TPUCFSR_T CFSR;
	TPUHSQR_T HSQR;
	TPUHSRR_T HSRR;
	TPUCPR_T  CPR;
	TPUCISR_T CISR;
	TPULR_T   LR;
	TPUSGLR_T SGLR;
	TPUDCNR_T DCNR;
	TPUMCR2_T TPUMCR2;
        uint8     BIG_HOLE[0xD6];
	TPUPRAM_T PRAM[16];
	
} TPU_T;


#endif 

// vim: ts=4
// vim: sw=4
